/**
   @file RAK5860_MQTT.ino
   @author rakwireless.com
   @brief  Examples to explain how to use MQTT related AT commands.
   @version 0.1
   @date 2023-3-14
   @copyright Copyright (c) 2023
**/

#include "Arduino.h"

String command; // String to store BG96 commands.
String result;	// String to store BG96 responses.

// Read the return value of BG96.
void BG96_read(time_t timeout)
{
	time_t timeStamp = millis();
	char c;
	result = "";

	while ((millis() - timeStamp) < timeout)
	{
		if (Serial1.available() > 0)
		{
			c = (char)Serial1.read();
			result = result + c;

			Serial.print((char)c);
			delay(1);
			if (result.indexOf("OK\r") != -1)
			{
				// Found an OK
				// Serial.println();
				break;
			}
		}
	}
}

// Write commands to BG96.
void BG96_write(const char *command)
{
	while (*command)
	{
		Serial1.write(*command);
		command++;
	}
	Serial1.println();
}

void setup()
{

	time_t serial_timeout = millis();
	Serial.begin(115200);
	while (!Serial)
	{
		if ((millis() - serial_timeout) < 5000)
		{
			delay(100);
		}
		else
		{
			break;
		}
	}

	// Check if the modem is already awake
	time_t timeout = millis();
	bool moduleSleeps = true;
	Serial1.begin(115200);
	delay(1000);
	Serial1.println("ATI");
	// BG96 init
	while ((millis() - timeout) < 4000)
	{
		if (Serial1.available())
		{
			String result = Serial1.readString();
			Serial.println("Modem response after start:");
			Serial.println(result);
			moduleSleeps = false;
		}
	}
	Serial.println("BG96 power up!");
	delay(1000);

	// Send "AT" to check BG96 is working
	command = "AT\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	// Enable echo
	command = "ATE\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	Serial.println("Sending AT+QSIMDET?");
	// Check SIM card detection status
	command = "AT+QSIMDET?\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	Serial.println("Sending AT+CIMI?");
	// Get SIM card IMEI
	command = "AT+CIMI\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	Serial.println("Sending AT+CPIN?");
	// Check SIM card pin protection
	command = "AT+CPIN?\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	command = "AT+CFUN=1,0\r";
	BG96_write(command.c_str());
	BG96_read(15000);

	command = "AT+CGATT=1\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+CEREG?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+CPIN?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QNWINFO\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QCSQ\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+CSQ\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+COPS=?\r";
	Serial.println("\nWaiting for the COPS response");
	BG96_write(command.c_str());
	BG96_read(40000);

	// Define PDP Context and APN
	command = "AT+CGDCONT=1,\"IP\",\"data.mono\"\r"; // Monogoto
	// command = "AT+CGDCONT=1,\"IP\",\"internet\"\r"; // Smart
	BG96_write(command.c_str());
	BG96_read(2000);

	// command = "AT+COPS=1,0,\"Globe\",0\r"; // Using Globe & configure the PDP context ID as 1.
	command = "AT+COPS=1,0,\"Smart\",0\r"; // Using SMART & configure the PDP context ID as 1.
	BG96_write(command.c_str());
	BG96_read(10000);

	Serial.println("\nSending second AT+QNWINFO");
	command = "AT+QNWINFO\r";
	BG96_write(command.c_str());
	BG96_read(5000);

	command = "AT+QMTOPEN=?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	// MQTT broker address
	command = "AT+QMTOPEN=0,\"45ce38183b7d4cfeb3bce15b3f2fc88a.s1.eu.hivemq.cloud\",8883\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTOPEN?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTCONN=?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	// MQTT clientid, username and password
	command = "AT+QMTCONN=0,\"bla\",\"bla\",\"bla\"\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTCONN?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTSUB=?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTSUB=0,1,\"#\",2\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTSUB=?\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTPUB=?\r";
	BG96_write(command.c_str());
	BG96_read(2000);
}

void loop()
{
	command = "AT+QMTPUB=0,0,0,0,\"topic/pub\"\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "Hello RAKWireless\r\032";
	BG96_write(command.c_str());
	BG96_read(2000);

	command = "AT+QMTSUB=0,1,\"#\",2\r";
	BG96_write(command.c_str());
	BG96_read(2000);

	delay(2000);
}
